/* Global core functions
 *
 * (C) 2003-2012 Anope Team
 * Contact us at team@anope.org
 *
 * Please read COPYING and README for further details.
 *
 * Based on the original code of Epona by Lara.
 * Based on the original code of Services by Andy Church.
 */

/*************************************************************************/

#include "module.h"
#include "global.h"

class CommandGLGlobal : public Command
{
 public:
	CommandGLGlobal(Module *creator) : Command(creator, "global/global", 1, 1)
	{
		this->SetDesc(_("Send a message to all users"));
		this->SetSyntax(_("\037message\037"));
	}

	void Execute(CommandSource &source, const std::vector<Anope::string> &params) anope_override
	{
		const Anope::string &msg = params[0];

		if (!global)
			source.Reply("No global reference, is gl_main loaded?");
		else
		{
			Log(LOG_ADMIN, source, this);
			global->SendGlobal(findbot(Config->Global), source.GetNick(), msg);
		}
	}

	bool OnHelp(CommandSource &source, const Anope::string &subcommand) anope_override
	{
		this->SendSyntax(source);
		source.Reply(" ");
		source.Reply(_("Allows Administrators to send messages to all users on the \n"
				"network. The message will be sent from the nick \002%s\002."), source.owner->nick.c_str());
		return true;
	}
};

class GLGlobal : public Module
{
	CommandGLGlobal commandglglobal;

 public:
	GLGlobal(const Anope::string &modname, const Anope::string &creator) : Module(modname, creator, CORE),
		commandglglobal(this)
	{
		this->SetAuthor("Anope");

	}
};

MODULE_INIT(GLGlobal)
